#include <Wire.h>

#include "SparkFunCCS811.h"
#define CCS811_ADDR 0x5B //Default I2C Address

#define PPM_LIMIT      2000   // 1000
#define ALERT_INTERVAL  300   // 300 second
#define ALERT_PIN        16   // WeMosD1Mini - D0
#define REFRESH_TIME   3600   // 1 hour

CCS811 mySensor(CCS811_ADDR);
uint16_t co2;
uint16_t elapsed;
uint16_t repeat;

#include "ErOled.h"
ErOled er_oled;
uint8_t msg[16];

void setup()
{
	pinMode(ALERT_PIN, OUTPUT);
	digitalWrite(ALERT_PIN,LOW);
	elapsed = 0;
	repeat  = 0;

	Wire.begin(); //Inialize I2C Hardware

	er_oled.begin();
	er_oled.rotate(2);

	if (mySensor.begin() == false) while(1);
}

void loop()
{
	if (mySensor.dataAvailable())
	{
		mySensor.readAlgorithmResults();
		co2 = mySensor.getCO2();
		sprintf((char *)&msg,"%d",co2);
		er_oled.clear();
		er_oled.stack(12, 2, (char*)&msg, 16);
		er_oled.stack(32, 18, "ppm", 12);
		er_oled.display();
		if (co2 >= PPM_LIMIT) {
			repeat++;
			if ((repeat >= 6)&&(elapsed >= ALERT_INTERVAL)) {
				digitalWrite(ALERT_PIN,HIGH);
				delay(100);
				digitalWrite(ALERT_PIN,LOW);
				elapsed = 0;
			}
		} else {
			repeat   = 0;
		}
	}
	delay(10000);
	elapsed += 10;
	if (elapsed >= REFRESH_TIME) {
		mySensor.begin();
		elapsed = 0;
	}
}
