/***************************************************
//Web: http://www.buydisplay.com
EastRising Technology Co.,LTD
****************************************************/
#include "ErOled.h"

ErOled::ErOled() {
}

ErOled::~ErOled() {}

void ErOled::I2C_Write_Byte(uint8_t value, uint8_t Cmd)
{
  uint8_t Addr = 0x3c;
  Wire.beginTransmission(Addr);
  Wire.write(Cmd);
  Wire.write(value);
  Wire.endTransmission();
}

void ErOled::command(uint8_t Reg)
{
	I2C_Write_Byte(Reg,  0x00);
}

void ErOled::data(uint8_t Data)
{
		I2C_Write_Byte(Data, 0x40);
}

void ErOled::begin()
{
	if (ER_OLED_RST != -1) {
		pinMode(ER_OLED_RST, OUTPUT);      
		digitalWrite(ER_OLED_RST, HIGH);
		delay(10);
		digitalWrite(ER_OLED_RST, LOW);
		delay(10);
		digitalWrite(ER_OLED_RST, HIGH);
	}

	command(0xAE); // Display Off (sleep mode)

	command(0xD5); // 表示クロック分周率/発振周波数
	command(0x80); // 分周率:1/発振周波数:RESET

	command(0xA8); // 解像度
	command(0x1F); // 32MUX(1/32 duty)

	command(0xD3); // 垂直方向表示オフセット
	command(0x00); // not offset

	command(0x8D); // set Charge Pump enable/disable
	command(0x14); // Charge Pump Mode 7.5V(RESET)
	
	command(0x40); // 描画開始Line 0

	command(0xA6); // A6:標準表示/A7:白黒反転

	command(0xA4); // RAMの内容を反映する

	command(0xA1); // 水平描画方向 A0:右→左/A1:左→右

	command(0xC8); // 垂直描画方向 C0:上→下/C8:下→上

	command(0xDA); // 表示モード設定
	command(0x12); // 0x02:Alternative/0x12:Sequential

	command(0x81); // コントラスト制御(Double byte command)
	command(0xCF); // valid range 01h to FFh

	command(0xD9); // Set Pre-charge Period
	command(0xF1);

	command(0xDB); // Set Vcomh Deselect Level
	command(0x40); // - 0.65 x Vcc

	command(0xAF); // Display On


	width    = ER_OLED_WIDTH;
	height   = ER_OLED_HEIGHT;
	pages    = height / 8;
	oled_buf = (uint8_t*)malloc(width * height / 8);

	set_mode(ER_MODE_NORMAL);
}

void ErOled::set_mode(uint8_t Mode)
{
	mode = Mode;
}

void ErOled::rotate(uint8_t dir)
{
	// 180°回転
	command(0xA0);	// 水平描画方向：左から右
	command(0xC0);	// 垂直描画方向：上から下
}

void ErOled::clear()
{
	for(int i = 0;i < width * height / 8;i++) oled_buf[i] = 0;
}

void ErOled::pixel(int x, int y, char color)
{
    if(x > width || y > height)return ;
    if(color) oled_buf[x+(y/8)*width] |= 1<<(y%8);
    else      oled_buf[x+(y/8)*width] &= ~(1<<(y%8));
}

void ErOled::char1616(uint8_t x, uint8_t y, uint8_t chChar)
{
	uint8_t i, j;
	uint8_t chTemp = 0, y0 = y, chMode = 0;

	for (i = 0; i < 32; i++) {
		chTemp = pgm_read_byte(&Font1612[chChar - 0x30][i]);
		for (j = 0; j < 8; j++) {
			chMode = chTemp & 0x80? 1 : 0; 
			ErOled::pixel(x, y, chMode);
			chTemp <<= 1;
			y++;
			if ((y - y0) == 16) {
				y = y0;
				x++;
				break;
			}
		}
	}
}

void ErOled::stack_char(unsigned char x, unsigned char y, char acsii, char size)
{
	unsigned char i, j, y0=y;
	char temp;
	unsigned char ch = acsii - ' ';
	for(i = 0;i<size;i++) {
		if(size == 12) {
			if(mode) temp = pgm_read_byte(&Font1206[ch][i]);
			else     temp = ~pgm_read_byte(&Font1206[ch][i]);
		} else {
			if(mode)temp = pgm_read_byte(&Font1608[ch][i]);
			else    temp = ~pgm_read_byte(&Font1608[ch][i]);
		}
		for(j =0;j<8;j++) {
			if(temp & 0x80) pixel(x, y, 1);
			else pixel(x, y, 0);
			temp <<= 1;
			y++;
			if((y-y0) == size) {
				y = y0;
				x++;
				break;
			}
		}
	}
}

void ErOled::stack(uint8_t x, uint8_t y, const char *pString, uint8_t Size)
{
	while (*pString != '\0') {       
		if (x > (width - Size / 2)) {
			x = 0;
			y += Size;
			if (y > (height - Size)) y = x = 0;
		}
		stack_char(x, y, *pString, Size);
		x += Size / 2;
		pString++;
	}
}

void ErOled::print(uint8_t x, uint8_t y, const char *pString, uint8_t Size)
{
	clear();
	stack(x, y, pString, Size);
	display();
}

void ErOled::char3216(uint8_t x, uint8_t y, uint8_t chChar)
{
	uint8_t i, j;
	uint8_t chTemp = 0, y0 = y, chMode = 0;

	for (i = 0; i < 64; i++) {
		chTemp = pgm_read_byte(&Font3216[chChar - 0x30][i]);
		for (j = 0; j < 8; j++) {
			chMode = chTemp & 0x80? 1 : 0; 
			pixel(x, y, chMode);
			chTemp <<= 1;
			y++;
			if ((y - y0) == 32) {
				y = y0;
				x++;
				break;
			}
		}
	}
}

void ErOled::bitmap(uint8_t x,uint8_t y,const uint8_t *pBmp, uint8_t chWidth, uint8_t chHeight)
{
	uint8_t i, j, byteWidth = (chWidth + 7)/8;
	for(j = 0;j < chHeight;j++){
		for(i = 0;i <chWidth;i++){
			if(pgm_read_byte(pBmp + j * byteWidth + i / 8) & (128 >> (i & 7))){
				pixel(x + i,y + j, 1);
			}
		}
	}		
}

void ErOled::display()
{
	uint8_t page,i;   
	for (page = 0; page < pages; page++) {
		command(0xB0 + page); /* set page address        */
		command(0x00);        /* set low column address  */
		command(0x12);        /* set high column address */
		for(i = 0; i< width; i++ ) {
			data(oled_buf[i+page*width]); // write data one
		}
	}
}
